/*
	Filename	Lorentz.c
	Copyright  1993 David Hart. All rights reserved.
	Author		David Hart
	Description	Lorentz module for After Dark
	
	Version	  Date		Comments
	2.00	27/10/93	Original version for After Dark version 2.0
*/
 
#include <QuickDraw.h>
#include <Memory.h>
#include <OSUtils.h>
#include "GraphicsModule_Types.h"

/* Function prototypes */

long	Colors[] = {
	/*	blackColor,	*/
		whiteColor,
		redColor,
		greenColor,
		blueColor,
		cyanColor,
		magentaColor,
		yellowColor };

#define MAX	7000	/* Max number of points */

/* Specific graphics module data structures */
typedef struct MyStorage
	{
	int		new;
	double	z;
	double	X[MAX];
	double	Y[MAX];
	} MyStorage, *MyStoragePtr, **MyStorageHandle;

/*
	DoInitialize is the first function called from After Dark.
	Memory for the structure is allocated and the variables are initialized.
	The allocated memory is assigned to "storage"
	and the function returns "noErr" if there are no problems.
*/
OSErr	DoInitialize(Handle *storage, RgnHandle blankRgn, GMParamBlockPtr params)
	{
	MyStorageHandle myStorage;
	int	t;
	
	/* allocate handle to my storage */
	myStorage = (MyStorageHandle)NewHandle(sizeof(MyStorage));
	
	if (!myStorage)
		return MemError();
	
	MoveHHi(myStorage);
	HLock(myStorage);					/* Lock it down */
	
	*storage = (Handle)myStorage;		/* Assign myStorage to passed handle */
	
	(**myStorage).new = 0;
	(**myStorage).z = 1;
	for (t = 0; t < MAX; t++)
		{
		(**myStorage).X[t] = 1;
		(**myStorage).Y[t] = 1;
		}
	HUnlock(myStorage);
	return noErr;
	}

/*  
	DoBlank is the next function called.
	It is used to simply blank the screen.
*/ 
OSErr	DoBlank(Handle storage, RgnHandle blankRgn, GMParamBlockPtr params)
	{
	FillRgn(blankRgn, params->qdGlobalsCopy->qdBlack);
	return noErr;
	}

/*
	DoDrawFrame does almost all of the work.
*/
OSErr	DoDrawFrame(Handle storage, RgnHandle blankRgn, GMParamBlockPtr params)
	{
	OSErr err = noErr;
	MyStoragePtr myStorage;				/* to hold dereferenced storage handle */
	int	a = 10;
	int	b = 28;
	
	double	c = 8.0 / 3.0;
	double	dt = .01;
	double	dx;
	double	dy;
	double	dz;
	double	x;
	double	y;
	
	int	length;
	int	old;	

	int	width;
	int	height;
	int size;
	int	color;
	int	monitor;
	int	xm;
	int	ym;
	Rect	rect;

	/* Lock our storage down so we can dereference it once for faster access */
	MoveHHi(storage);
	HLock(storage);
	myStorage = (MyStoragePtr)*storage;
	
	/* Get params */
	a = params->controlValues[0] * 6 / 100 + 5;
	if (a > 10) a = 10;
	b = params->controlValues[1] * 9 / 100 + 20;
	if ( b > 28) b = 28;
	length = (long)(MAX-1) * (params->controlValues[2] + 1) / 101;
	
	/* Color */
	switch (params->controlValues[3])
		{
		case 8:	/* rainbow */
			color = myStorage->new % 7 + 1;
			break;
		case 9:	/* cycle */
			color = myStorage->new * 7 / MAX;
			break;
		default:
			color = params->controlValues[3] - 1;
			break;
		}
	
	/* For each monitor */
	for (monitor = 0; monitor < params->monitors->monitorCount; monitor++)
		{
		rect = params->monitors->monitorList[monitor].bounds;
		xm = (rect.left + rect.right) / 2;
		ym = (rect.top + rect.bottom) / 2;
		width = rect.right - rect.left;
		height = rect.bottom - rect.top;
		size = width > height ? width : height;
		
		/* Remove old */
		old = (myStorage->new + MAX - length) % MAX;
		x = (myStorage->X)[old];
		y = (myStorage->Y)[old];
		MoveTo(x * width / 44 + xm, y * height / 60 + ym);
		
		old = (old+1) % MAX;
		x = (myStorage->X)[old];
		y = (myStorage->Y)[old];
		ForeColor(blackColor);
		LineTo(x * width / 44 + xm, y * height / 60 + ym);
		
		/* Move to last new */
		x = (myStorage->X)[myStorage->new];
		y = (myStorage->Y)[myStorage->new];
		MoveTo(x * width / 44 + xm, y * height / 60 + ym);
		
		/* Calculate new position */
		dx = a * (y - x);
		dy = b * x - y - (x * myStorage->z);
		dz = (x * y) - (c * myStorage->z);
		
		x += dx * dt;
		y += dy * dt;
		myStorage->z += dz * dt;
		
		/* Save new position */
		myStorage->new = (myStorage->new+1) % MAX;
		(myStorage->X)[myStorage->new] = x;
		(myStorage->Y)[myStorage->new] = y;
		
		/* Move to new position */
		ForeColor(Colors[color]);
		LineTo(x * width / 44 + xm, y * height / 60 + ym);
		}
	
	HUnlock(storage);
	return noErr;
	}

/*
	DoClose is called when the user quits the screen saver.
	The memory is deallocated and the function returns "MemError"
	which will tell After Dark if there was a problem
*/
OSErr	DoClose(Handle storage, RgnHandle blankRgn, GMParamBlockPtr params)
	{
	MyStorageHandle myStorage = (MyStorageHandle)storage;
		
	/* Deallocate our storage */
	if (myStorage)
		{
		MoveHHi(myStorage);
		HLock(myStorage);
		DisposHandle(storage);
		}
	
	/* check for memory errors */
	return MemError();
	}

/*
	DoSetUp is called if the user clicks on a button in the Control Panel.
*/
OSErr	DoSetUp(RgnHandle blankRgn, short message, GMParamBlockPtr params)
	{
	return noErr;
	}
